*!v1.0 (28/6/2021)
*!Authors: Koen Jochmans and Vincenzo Verardi
program define ivnet, eclass

    * This program is free software: you can redistribute it and/or modify it.

	* This program is distributed in the hope that it will be useful,
	* but WITHOUT ANY WARRANTY; without even the implied warranty of
	* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
	* GNU General Public License for more details <http://www.gnu.org/licenses/>

    version 13.0
	*preserve
	tempvar touse
    ///cap mata: mata set matafavor speed

    if replay()& "`e(cmd)'"=="ivnet" {
        ereturn  display
        exit
    }

    syntax varlist,  cluster(string) [path(string) iterations(integer 4) stub(string)]

    capt mata mata which mm_which()
    if _rc {
        di as error "-moremata- is required; type -ssc install moremata- to obtain it"
        exit 499
    }

	if "`path'"=="" {
		local path "`c(pwd)'"
	}
	
	mark `touse'
	tempvar ivnet_G ivnet_nG
	if `iterations' <=0 {
		noi di in r "iterations must be strictly positive"
		exit 411
	}

	local dv: word 1 of `varlist'
    local exp: list varlist -dv

	qui mata: mata matuse `path'/ivnet_A, replace

	bysort `cluster': gen `ivnet_nG'=_N
	qui sum `ivnet_nG'

	if r(min)<=1 {
		di ""
		noi di in r "Error: all clusters should contain at least two individuals"
		exit
	}


	mata: ivnet_y=asarray_create("real",1)
	mata: ivnet_x=asarray_create("real",1)
	egen `ivnet_G'=group(`cluster')
	qui sum `ivnet_G'
	local max=r(max)

	forvalues ivnet_i=1(1)`max' {
		qui putmata ivnet_x0=(x*) if `ivnet_G'==`ivnet_i', replace
		qui putmata ivnet_yo=(y) if `ivnet_G'==`ivnet_i', replace
		mata: asarray(ivnet_x,`ivnet_i',ivnet_x0)
		mata: asarray(ivnet_y,`ivnet_i',ivnet_yo)

	}

	///mata: construct("`varlist'","`touse'")

	mata { 

		ivnet_G=`max'
		ivnet_nlags=`iterations'*cols(ivnet_x0)
		ivnet_zfin=J(0,ivnet_nlags,.)
		ivnet_Hxfin=J(0,cols(ivnet_x0),.)
		ivnet_Hyfin=J(0,1,.)
		for (ivnet_g=1;ivnet_g<=ivnet_G;ivnet_g++) {
			ivnet_Ag=asarray(ivnet_A,ivnet_g)
			ivnet_yg=asarray(ivnet_y,ivnet_g)
			ivnet_zgfin=J(rows(ivnet_Ag),0,.)
			ivnet_Hxgfin=J(rows(ivnet_Ag),0,.)

			for (ivnet_k=1;ivnet_k<=cols(ivnet_x0);ivnet_k++) {
				ivnet_xg=asarray(ivnet_x,ivnet_g)[,ivnet_k]
				ivnet_zg=J(rows(ivnet_xg),`iterations',.)
				ivnet_Hg = ivnet_Ag:/rowmax((J(rows(ivnet_Ag),1,1),rowsum(ivnet_Ag))); ivnet_Hx=ivnet_Hg*ivnet_xg;
				ivnet_Hxgfin=(ivnet_Hxgfin,ivnet_Hx)

				///Instrument creation

				for (ivnet_i=1;ivnet_i<=rows(ivnet_Ag);ivnet_i++) {    
					ivnet_row=1::rows(ivnet_Ag)
					ivnet_Ai=ivnet_Ag[select(ivnet_row, ivnet_row:!=ivnet_i),select(ivnet_row, ivnet_row:!=ivnet_i)]; ivnet_Hi = ivnet_Ai:/rowmax((J(rows(ivnet_Ai),1,1),rowsum(ivnet_Ai)))
					ivnet_xi=ivnet_xg[select(ivnet_row, ivnet_row:!=ivnet_i),]

					ivnet_Hxi=(ivnet_Hi*ivnet_xi);
					ivnet_zg[ivnet_i,1]=mean(ivnet_Hxi)

					if (`iterations'>=2) {
						for(ivnet_l=2;ivnet_l<=`iterations';ivnet_l++) {
							ivnet_Hxi=ivnet_Hi*ivnet_Hxi
							ivnet_zg[ivnet_i,ivnet_l]=mean(ivnet_Hxi)
						}
					}

				}

				ivnet_zgfin=(ivnet_zgfin,ivnet_zg)
			}
			ivnet_Hy=ivnet_Hg*ivnet_yg;
			ivnet_zfin=(ivnet_zfin\ivnet_zgfin)
			ivnet_Hyfin=(ivnet_Hyfin\ivnet_Hy)
			ivnet_Hxfin=(ivnet_Hxfin\ivnet_Hxgfin)

		}
	}


	getmata (`stub'_inst*)=ivnet_zfin
	getmata ivnet_Hy=ivnet_Hyfin
	getmata (ivnet_Hx*)=ivnet_Hxfin


	foreach var in varlist ivnet_* {
		capture mata: mata drop `var'
	}


	ivregress 2sls `varlist' (ivnet_Hy ivnet_Hx*=`stub'_inst*), cluster(`cluster')

end

exit
