{smcl}
{* *! version 1.0  02/07/2021}{...}
{cmd:help ivnet} {right: (Koen Jochmans and Vincenzo Verardi)}
{hline}

{title:Title}

{p2colset 5 14 20 2}{...}
{p2col:{hi:ivnet} {hline 1}}{title:Self-selection of peers in the linear-in-means model (Jochmans, 2020)}{p_end}
{p2colreset}{...}

{title:Syntax}

{p 8 12 2}
{cmd:ivnet} {varlist} {cmd:,} {opt cluster(varname)} [{opt path(location)} iterations(#) stub(string)]

{synoptset 19 tabbed}
{marker semiparopts}{...}
{synopthdr}
{synoptline}

{syntab:Model}
{synopt:{opt cluster(varname)}}Specifies the cluster indicator{p_end}
{synopt:{opt path(location)}}Specifies the location where the ivnet_A associative array is stored (if not in the current directory) {p_end}
{synopt:{opt iterations(#)}}Specifies the number of iterations used to construct the instruments (4 by default){p_end}
{synopt:{opt stub(string)}}Specifies a prefix used for the instruments{p_end}

{synoptline}
{p2colreset}{...}

{title:Description}

{bf:ivnet} Implements a two-stage least-squares estimator for the linear-in-means model that allows for peer group membership to be endogenous. Endogeneity of the (row-normalised) adjacency matrix, H, is dealt with by constructing instruments based on leave-own-out networks. See Jochmans (2020) for details.


The command creates endogenous spillover effects (e.g. Hx), endogenous peer effects (Hy), and leave-own-out instruments. It subsequently runs 2sls and reports the results.

In addition to dependent and explanatory variables, an associative array of adjacency matrices (with 0 and 1 values) is required as input. Each "layer" in the adjacency matrix is associated to a cluster defined in the cluster option. This array MUST be called ivnet_A and stored on the hard drive. The location of this array is declared in the option "path".

{bf:Warning: }individuals within clusters should be sorted in the same order as they appear in rows and columns of 
the adjacency matrix. The adjacency matrices themselves should appear in the same order  as clusters in the cluster 
variable (for example for the first cluster, the adjacency matrix should be  the first in the array 
(i.e. asarray(ivnet_A,1)).


{title:Example}

Let's start with an empty dataset

{phang2}{cmd:.} {bf:{stata "clear"}} // warning, this command removes data from memory {p_end}

Change the working directory to what is wanted (in this example DGP was saved in c:/data). This is not compulsory 
as the location of where the ivnet_A is stored could be declared in the "path" option (eg. path(c:/data) here)

{phang2}{cmd:.} {bf:{stata "cd c:\data"}}{p_end}

Generate the data by running the DGP do file

{phang2}{cmd:.} {bf:{stata "do DGP.do"}}{p_end}

Run the model

{phang2}{cmd:.} {bf:{stata "ivnet y x*, cluster(G0) iterations(3)"}}{p_end}

After the instruments and the transition matrix of a random walk through the network have been generated, the code runs estimation

{phang2}{cmd:.} {bf:{stata "ivregress 2sls y x* (ivnet_H*=_inst*), cluster(G0)"}}{p_end}

where ivnet_H* are the average values in the neighborhood of both dependent and explanatory variables for each individual. 

This command could be substituted by the other instrumental vaariable commands available in Stata (eg. ivreg2, ivregress gmm, etc.)

{phang2}{cmd:.} {bf:{stata "ivreg2 y x* (ivnet_H*=_inst*), cluster(G0) gmm2s"}}{p_end}

Naturally, a partial model where the value of y in the neighborhood of an individual (endogenous peer effects) is not considered
could de fitted as well

{phang2}{cmd:.} {bf:{stata "ivreg2 y x* (ivnet_Hx*=_inst*), cluster(G0) gmm2s"}}{p_end}

{pstd}


{title:Authors}

{pstd}Koen Jochmans, University of Toulouse Capitole; and Vincenzo Verardi, FNRS-UNamur{p_end}
{pstd} Contact: Koen Jochmans <koen.jochmans@tse-fr.eu> {p_end}


{title:Also see}

{p 7 14 2}Help:  {helpb ivreg2} (if installed){p_end}

{title:References}

{phang}Jochmans, K. (2020). Peer effects and endogenous social interactions. {it:Mimeo}. https://arxiv.org/abs/2008.07886

{title:Remark}

Creating an array from available adjacency matrices is simple in mata. Let us assume that we want to create 
an array from  3 matrices.

We generate three matrices here for the example

{phang2}{cmd:.} {bf:{stata "mata: A1=runiform(10,2)"}}{p_end}
{phang2}{cmd:.} {bf:{stata "mata: A2=runiform(10,2)"}}{p_end}
{phang2}{cmd:.} {bf:{stata "mata: A3=runiform(10,2)"}}{p_end}

we now create an empty array called ivnet_A as required in the code

{phang2}{cmd:.} {bf:{stata `"mata: ivnet_A=asarray_create("real",1)"'}}{p_end}

we then loop over the matrices and accumulate them. 

To put the A1 matrix in the first layer we type

{phang2}{cmd:.} {bf:{stata "mata: asarray(ivnet_A,1,A1)"}}{p_end}

To put the A2 matrix in the second layer we type

{phang2}{cmd:.} {bf:{stata "mata: asarray(ivnet_A,2,A2)"}}{p_end}

To put the A3 matrix in the third layer we type

{phang2}{cmd:.} {bf:{stata "mata: asarray(ivnet_A,3,A3)"}}{p_end}

Naturally all this can be done with a simple "for" loop.

If we want to recover the matrix of a given layer (for example layer 2) we simply type

{phang2}{cmd:.} {bf:{stata "mata: asarray(ivnet_A,2)"}}{p_end}

The associative array can be saved (in the current directory) by typing

{phang2}{cmd:.} {bf:{stata "mata: mata matsave ivnet_A ivnet_A, replace "}}{p_end}

It can the be loaded by typing

{phang2}{cmd:.} {bf:{stata "mata: mata matuse ivnet_A, replacemata: mata matuse ivnet_A, replace "}}{p_end}